from .manager import Manager

def install(manager: Manager, directive_params: list, params: dict):
    if not params:
        deps_type = 'default'
        packages = directive_params
    elif directive_params:
        raise Exception('Invalid syntax\nUse `ezdeps -h` to watch command list')
    elif ('S' in params or 'default' in params):
        deps_type = 'default'
        packages = params['S'] if 'S' in params else params['default']
    elif ('D' in params or 'development' in params):
        deps_type = 'development'
        packages = params['D'] if 'D' in params else params['development']
    elif ('P' in params or 'peer' in params):
        deps_type = 'peer'
        packages = params['P'] if 'P' in params else params['peer']
    elif ('A' in params or 'all' in params):
        deps_type = 'all'
        packages = None

        other_params = params['A'] if 'A' in params else params['all']
        if other_params != True:
            raise Exception('Unexpected params')

    if packages == True:
        packages = None

    manager.install(packages, deps_type)



import os
from shutil import rmtree
from .utils.constants import CUR_DIR
from .manager import Manager

def publish(manager: Manager, params):
    clear_after = False
    if ('clear' in params or 'C' in params):
        clear_after = True
        if 'clear' in params:
            del params['clear']
        else:
            del params['C']

    params_array = []
    for key, values in params.items():
        if len(key) == 1:
            params_array.append(f'-{key}')
        else:
            params_array.append(f'--{key}')

        params_array.extend(values)

    params_line = ' '.join(params_array)
    os.system(f'twine upload {params_line} dist/*')

    if clear_after:
        rmtree(f'{CUR_DIR}/build')
        rmtree(f'{CUR_DIR}/dist')
        rmtree(f'{CUR_DIR}/{manager.name}.egg-info')



import re
from os import system
from pkg_resources import get_distribution

def parse_version(version: str):
    """
    Returns parsed version from .ezdeps.json
    """

    eq_type = '=='
    if version[0] == '^':
        eq_type = r'\>='
        version = version[1:]

    return eq_type + version

def parse_packages(packages: list):
    """
    Returns parsed packages from cli
    """

    regexp = re.compile(r'^([a-zA-Z_-]*)(?:@([^#]*))?(?:#(.*))?$')
    parsed_packages = {}

    for package in packages:
        match = regexp.search(package)
        
        if not match:
            raise Exception(f'Cannot parse package `{package}`')

        parsed_packages[match[1]] = {
            'version': match[2] and parse_version(match[2]),
            'source': match[3],
            'deps_version': match[2]
        }

    return parsed_packages

def use_auth(source: str, auth_data: dict = {}):
    def auth_provider(match):
        domain = match.group(1)

        auth_string = ''
        if domain in auth_data:
            domain_data = auth_data[domain]
            auth_string = f'{domain_data["USER"]}:{domain_data["PASSWORD"]}@'

        return f'//{auth_string}{domain}/'

    domain_regexp = r'//([^/]*)'
    return re.sub(domain_regexp, auth_provider, source)

def manage_package(
    command: str,
    name: str,
    package: dict = None,
    auth_data: dict = {},
    deps = True
):
        version = package and package['version']
        source = package and 'source' in package and package['source']
        if source:
            source = use_auth(source, auth_data)

        req_line = name if not version else f'{name}{version}'
        if source:
            req_line = f'--index-url {source} --no-deps {req_line}'
        
        if system(f'python3 -m pip {command} {req_line}'):
            raise Exception('Error installing pip module')
        
        if deps:
            try:
                requirements = ([str(r) for r in get_distribution(name).requires()])

                for requirement in requirements:
                    req_line = requirement.replace('>', r'\>').replace('<', r'\<')
                    if system(f'python3 -m pip install {req_line}'):
                        raise Exception('Error installing pip module')

            except Exception:
                pass
        
        try:
            return get_distribution(name).version
        except Exception:
            return



from .manager import Manager

def update(manager: Manager, directive_parmas: list, params: dict = None):
    if params:
        raise Exception('Unexpacted params')

    manager.update(directive_parmas)
